
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.007.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "ExtUtils::MakeMaker" => 0,
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "A client for the Bugzilla web services API.",
  "dist_author" => [
    "Dean Hamstead <dean\@bytefoundry.com.au>",
    "Jochen Wiedmann <jochen.wiedmann\@gmail.com>"
  ],
  "dist_name" => "BZ-Client",
  "dist_version" => "4.4001_001",
  "license" => "perl",
  "module_name" => "BZ::Client",
  "recursive_test_files" => 1,
  "requires" => {
    "DateTime::Format::ISO8601" => 0,
    "DateTime::Format::Strptime" => 0,
    "DateTime::TimeZone" => 0,
    "Encode" => 0,
    "File::Spec" => 0,
    "HTTP::CookieJar" => 0,
    "HTTP::Tiny" => 0,
    "URI" => 0,
    "XML::Parser" => 0,
    "XML::Writer" => 0,
    "parent" => 0,
    "perl" => "v5.8.0",
    "strict" => 0,
    "warnings" => 0
  },
  "test_requires" => {
    "Clone" => 0,
    "Data::Dumper" => 0,
    "DateTime" => 0,
    "Test::More" => 0,
    "Test::RequiresInternet" => 0,
    "Text::Password::Pronounceable" => 0
  }
);


my %fallback_build_requires = (
  "Clone" => 0,
  "Data::Dumper" => 0,
  "DateTime" => 0,
  "Module::Build" => "0.28",
  "Test::More" => 0,
  "Test::RequiresInternet" => 0,
  "Text::Password::Pronounceable" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
